! Copyright (C) 2010  ARPA-SIM <urpsim@smr.arpa.emr.it>
! authors:
! Davide Cesari <dcesari@arpa.emr.it>
! Paolo Patruno <ppatruno@arpa.emr.it>

! This program is free software; you can redistribute it and/or
! modify it under the terms of the GNU General Public License as
! published by the Free Software Foundation; either version 2 of 
! the License, or (at your option) any later version.

! This program is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
! GNU General Public License for more details.

! You should have received a copy of the GNU General Public License
! along with this program.  If not, see <http://www.gnu.org/licenses/>.


! to translate from C and use here ......
!#define NC_ERR_READ(nc_status) \
!    if (nc_status != NC_NOERR) { \
!        fprintf(stderr, \
!            "%s: Error reading netCDF file at line %d : %s \n",  ProgName, __LINE__, nc_strerror(nc_status) \
!        ); \
!    exit(1);\
!    }



MODULE vol7d_netcdf_class

USE char_utilities
USE vol7d_class
USE geo_coord_class
USE datetime_class
use netcdf
use log4fortran

IMPLICIT NONE
PRIVATE
PUBLIC  import, export

character (len=255),parameter:: subcategory="vol7d_netcdf_class"

!!$!>\brief importa
!!$INTERFACE import
!!$  MODULE PROCEDURE vol7d_netcdf_import
!!$END INTERFACE

!>\brief esporta
INTERFACE export
  MODULE PROCEDURE vol7d_netcdf_export
END INTERFACE



CONTAINS



subroutine vol7d_netcdf_export (this,ncconventions,ncunit,description,filename)

TYPE(vol7d),INTENT(IN) :: this !< volume vol7d da scrivere 
integer,optional,intent(inout) :: ncunit !< unit netcdf su cui scrivere; se passata =0 ritorna il valore rielaborato (default =elaborato internamente da netcdf )
character(len=*),intent(in) :: ncconventions !< tipo di convenzione da utilizzare nella scrittura netcdf (per ora supportato "CF-1.1 vol7d")
character(len=*),intent(inout),optional :: filename !< nome del file su cui scrivere; se passato ="" ritorna il valore rielaborato
character(len=*),INTENT(IN),optional :: description !< descrizione del volume

integer :: lunit
character(len=254) :: ldescription,arg,lfilename
integer :: nana, ntime, ntimerange, nlevel, nnetwork, &
 ndativarr, ndativari, ndativarb, ndativard, ndativarc,&
 ndatiattrr, ndatiattri, ndatiattrb, ndatiattrd, ndatiattrc,&
 ndativarattrr, ndativarattri, ndativarattrb, ndativarattrd, ndativarattrc,&
 nanavarr, nanavari, nanavarb, nanavard, nanavarc,&
 nanaattrr, nanaattri, nanaattrb, nanaattrd, nanaattrc,&
 nanavarattrr, nanavarattri, nanavarattrb, nanavarattrd, nanavarattrc
!integer :: im,id,iy
integer :: tarray(8)
logical :: opened,exist

integer :: ana_ident_varid,ana_dimid,ana_lon_varid,ana_lat_varid &
 ,ident_len_dimid,var_len_dimid &
 ,level_dimid,level_vdim_dimid,level_vect_varid,network_dimid,network_name_varid &
 ,network_name_len_dimid,timerange_vdim_dimid &
 ,time_iminuti_varid,time_dimid,timerange_dimid,timerange_vect_varid,var_vdim_dimid &
 ,dativard_dimid,dativarr_dimid,dativari_dimid,dativarb_dimid,dativarc_len_dimid,dativarc_dimid &
 ,voldativarr_varid ,voldativari_varid ,voldativard_varid ,voldativarb_varid ,voldativarc_varid &
 ,anavard_dimid,anavarr_dimid,anavari_dimid,anavarb_dimid,anavarc_len_dimid,anavarc_dimid &
 ,volanavarr_varid ,volanavari_varid ,volanavard_varid ,volanavarb_varid ,volanavarc_varid &
 ,anavarr_varid,anavari_varid,anavard_varid,anavarb_varid,anavarc_varid &
 ,dativarr_varid,dativari_varid,dativard_varid,dativarb_varid,dativarc_varid


integer :: i

type(datetime) :: timeref 
character (len=23) :: isodate


integer :: category
character(len=512):: a_name

character(len=65):: varchar(3)

call l4f_launcher(a_name,a_name_append=subcategory)
category=l4f_category_get(a_name)


if (ncconventions == "CF-1.1") then
   call vol7d_netcdf_export_CF (this,ncconventions,ncunit,description,filename)
else if (ncconventions /= "CF-1.1 vol7d") then

  call l4f_category_log(category,L4F_FATAL,"ncconventions not supported: "// &
   trim(ncconventions))
  call raise_fatal_error()
end if

call date_and_time(values=tarray)
call getarg(0,arg)

if (present(description))then
  ldescription=description
else
  ldescription="NETCDF generated by: "//trim(arg)
end if


lfilename=trim(arg)//".nc"
if (index(arg,'/',back=.true.) > 0) lfilename=lfilename(index(arg,'/',back=.true.)+1 : )

if (present(filename))then
  if (filename == "")then
    filename=lfilename
  else
    lfilename=filename
  end if
end if

IF (PRESENT(ncunit)) THEN
  lunit = ncunit
ELSE
  lunit = 0
ENDIF

IF (lunit == 0) THEN

  INQUIRE(file=lfilename,EXIST=exist)
  IF (exist) THEN 
    CALL l4f_category_log(category,L4F_ERROR, &
     "file exists, cannot open file "//TRIM(lfilename))
    CALL raise_fatal_error()
  END IF

  CALL nccheck( "0",nf90_create(lfilename, nf90_clobber, lunit) )
  CALL l4f_category_log(category,L4F_INFO, "opened "//TRIM(lfilename))

END IF
IF (PRESENT(ncunit)) ncunit = lunit ! reassign for output

call init(timeref,year=1,month=1,day=1,hour=00,minute=00)
call getval(timeref,isodate=isodate)

nana=size(this%ana)
ntime=size(this%time)
ntimerange=size(this%timerange)
nlevel=size(this%level)
nnetwork=size(this%network)

ndativarr=0
ndativari=0
ndativarb=0
ndativard=0
ndativarc=0

if (associated(this%dativar%r)) ndativarr=size(this%dativar%r)
if (associated(this%dativar%i)) ndativari=size(this%dativar%i)
if (associated(this%dativar%b)) ndativarb=size(this%dativar%b)
if (associated(this%dativar%d)) ndativard=size(this%dativar%d)
if (associated(this%dativar%c)) ndativarc=size(this%dativar%c)

ndatiattrr=size(this%datiattr%r)
ndatiattri=size(this%datiattr%i)
ndatiattrb=size(this%datiattr%b)
ndatiattrd=size(this%datiattr%d)
ndatiattrc=size(this%datiattr%c)

ndativarattrr=size(this%dativarattr%r)
ndativarattri=size(this%dativarattr%i)
ndativarattrb=size(this%dativarattr%b)
ndativarattrd=size(this%dativarattr%d)
ndativarattrc=size(this%dativarattr%c)

nanavarr=0
nanavari=0
nanavarb=0
nanavard=0
nanavarc=0
 
if (associated(this%anavar%r)) nanavarr=size(this%anavar%r)
if (associated(this%anavar%i)) nanavari=size(this%anavar%i)
if (associated(this%anavar%b)) nanavarb=size(this%anavar%b)
if (associated(this%anavar%d)) nanavard=size(this%anavar%d)
if (associated(this%anavar%c)) nanavarc=size(this%anavar%c)

nanaattrr=size(this%anaattr%r)
nanaattri=size(this%anaattr%i)
nanaattrb=size(this%anaattr%b)
nanaattrd=size(this%anaattr%d)
nanaattrc=size(this%anaattr%c)

nanavarattrr=size(this%anavarattr%r)
nanavarattri=size(this%anavarattr%i)
nanavarattrb=size(this%anavarattr%b)
nanavarattrd=size(this%anavarattr%d)
nanavarattrc=size(this%anavarattr%c)

!write(unit=lunit)ldescription
!write(unit=lunit)tarray


!call nccheck( "0Conventions",nf90_put_att(lunit, NF90_GLOBAL ,"Conventions",ncconventions))
call nccheck( "0Conventions",nf90_put_att(lunit, NF90_GLOBAL ,"Conventions","CF-1.1"))
call nccheck( "0title",nf90_put_att(lunit, NF90_GLOBAL ,"title", ldescription))


call nccheck( "1",nf90_def_dim(lunit,"ana", nana, ana_dimid) )
call nccheck( "2",nf90_def_dim(lunit,"ident_len",vol7d_ana_lenident , ident_len_dimid) )

call nccheck( "3",nf90_def_dim(lunit,"time", ntime, time_dimid) )

call nccheck( "4",nf90_def_dim(lunit,"timerange", ntimerange, timerange_dimid) )
call nccheck( "5",nf90_def_dim(lunit,"timerange_vdim", 3, timerange_vdim_dimid) )

call nccheck( "6",nf90_def_dim(lunit,"level", nlevel, level_dimid) )
call nccheck( "7",nf90_def_dim(lunit,"level_vdim", 4, level_vdim_dimid) )

call nccheck( "8",nf90_def_dim(lunit,"network_name", nnetwork, network_dimid) )
call nccheck( "9",nf90_def_dim(lunit,"network_name_len",network_name_len, network_name_len_dimid) )

call nccheck( "10",nf90_def_dim(lunit,"var_vdim",3, var_vdim_dimid) )
call nccheck( "11",nf90_def_dim(lunit,"var_len",65, var_len_dimid) )


if (nanavarr > 0) call nccheck( "a1",nf90_def_dim(lunit,"anavarr", nanavarr, anavarr_dimid) )
if (nanavari > 0) call nccheck( "a2",nf90_def_dim(lunit,"anavari", nanavari, anavari_dimid) )
if (nanavarb > 0) call nccheck( "a3",nf90_def_dim(lunit,"anavarb", nanavarb, anavarb_dimid) )
if (nanavard > 0) call nccheck( "a4",nf90_def_dim(lunit,"anavard", nanavard, anavard_dimid) )
if (nanavarc > 0) call nccheck( "a5",nf90_def_dim(lunit,"anavarc", nanavarc, anavarc_dimid) )
call nccheck( "a6",nf90_def_dim(lunit,"anavarc_len",vol7d_cdatalen, anavarc_len_dimid) )


if (ndativarr > 0) call nccheck( "d1",nf90_def_dim(lunit,"dativarr", ndativarr, dativarr_dimid) )
if (ndativari > 0) call nccheck( "d2",nf90_def_dim(lunit,"dativari", ndativari, dativari_dimid) )
if (ndativarb > 0) call nccheck( "d3",nf90_def_dim(lunit,"dativarb", ndativarb, dativarb_dimid) )
if (ndativard > 0) call nccheck( "d4",nf90_def_dim(lunit,"dativard", ndativard, dativard_dimid) )
if (ndativarc > 0) call nccheck( "d5",nf90_def_dim(lunit,"dativarc", ndativarc, dativarc_dimid) )
call nccheck( "d6",nf90_def_dim(lunit,"dativarc_len",vol7d_cdatalen, dativarc_len_dimid) )

! ripetere per datiattr anavar anaattr -- dativarattr anavarattr


call nccheck( "10",nf90_def_var(lunit, "ana_lat", NF90_DOUBLE, ana_dimid, ana_lat_varid) )
call nccheck( "10long_name",nf90_put_att(lunit,ana_lat_varid  ,"long_name","latitude") )
call nccheck( "10units",nf90_put_att(lunit,ana_lat_varid  ,"units","degrees_north") )
call nccheck( "10standard_name",nf90_put_att(lunit,ana_lat_varid  ,"standard_name","latitude") )
call nccheck( "10fillvalue",nf90_put_att(lunit,ana_lat_varid  ,"_FillValue",dmiss) )
call nccheck( "10missing_value",nf90_put_att(lunit,ana_lat_varid  ,"missing_value",dmiss) )

call nccheck( "11",nf90_def_var(lunit, "ana_lon", NF90_DOUBLE, ana_dimid, ana_lon_varid) )
call nccheck( "11long_name",nf90_put_att(lunit,ana_lon_varid  ,"long_name","longitude") )
call nccheck( "11units",nf90_put_att(lunit,ana_lon_varid  ,"units","degrees_east") )
call nccheck( "11standard_name",nf90_put_att(lunit,ana_lon_varid  ,"standard_name","longitude") )
call nccheck( "11fillvalue",nf90_put_att(lunit,ana_lon_varid  ,"_FillValue",dmiss) )
call nccheck( "11missing_value",nf90_put_att(lunit,ana_lon_varid  ,"missing_value",dmiss) )

call nccheck( "12",nf90_def_var(lunit, "ana_ident", NF90_CHAR, (/ ident_len_dimid,ana_dimid/), ana_ident_varid) )
!call nccheck( "12ident",nf90_put_att(lunit,ana_ident_varid  ,"long_name","ana_ident") )
call nccheck( "12fillvalue",nf90_put_att(lunit,ana_ident_varid  ,"_FillValue",cmiss) )
call nccheck( "12missing_value",nf90_put_att(lunit,ana_ident_varid  ,"missing_value",cmiss) )

call nccheck( "13",nf90_def_var(lunit, "time", NF90_INT, time_dimid, time_iminuti_varid) )
call nccheck( "1313",nf90_put_att(lunit,time_iminuti_varid, "units","minute since "//isodate) )
call nccheck( "13fillvalue",nf90_put_att(lunit,time_iminuti_varid  ,"_FillValue",imiss) )
call nccheck( "13missing_value",nf90_put_att(lunit,time_iminuti_varid  ,"missing_value",imiss) )

call nccheck( "14",nf90_def_var(lunit, "timerange", NF90_INT, (/timerange_vdim_dimid,timerange_dimid/), timerange_vect_varid) )
call nccheck( "14fillvalue",nf90_put_att(lunit, timerange_vect_varid ,"_FillValue",imiss) )
call nccheck( "14missing_value",nf90_put_att(lunit, timerange_vect_varid ,"missing_value",imiss) )

call nccheck( "15",nf90_def_var(lunit, "level", NF90_INT, (/level_vdim_dimid,level_dimid/), level_vect_varid) )
call nccheck( "15fillvalue",nf90_put_att(lunit,level_vect_varid  ,"_FillValue",imiss) )
call nccheck( "15missing_value",nf90_put_att(lunit,level_vect_varid  ,"missing_value",imiss) )

call nccheck( "16",nf90_def_var(lunit, "network_name", NF90_CHAR,(/ network_name_len_dimid,network_dimid/), network_name_varid) )
call nccheck( "16fillvalue",nf90_put_att(lunit,network_name_varid  ,"_FillValue",cmiss) )
call nccheck( "16missing_value",nf90_put_att(lunit,network_name_varid  ,"missing_value",cmiss) )


! anagrafica

if (nanavarr > 0)then
   call nccheck( "a81" ,nf90_def_var(lunit,"anavarr",NF90_CHAR,&
        (/var_len_dimid ,var_vdim_dimid,anavarr_dimid/),anavarr_varid ))
   call nccheck( "a81fillvalue",nf90_put_att(lunit,anavarr_varid ,"_FillValue",cmiss) )
   call nccheck( "a81missing_value",nf90_put_att(lunit,anavarr_varid ,"missing_value",cmiss) )

   call nccheck( "a82" ,nf90_def_var(lunit,"volanavarr",NF90_REAL,&
        (/ana_dimid,anavarr_dimid,network_dimid/),volanavarr_varid ))
   call nccheck( "a82fillvalue",nf90_put_att(lunit,volanavarr_varid  ,"_FillValue",rmiss) )
   call nccheck( "a82missing_value",nf90_put_att(lunit,volanavarr_varid  ,"missing_value",rmiss) )
end if
if (nanavari > 0) then
   call nccheck( "a83" ,nf90_def_var(lunit,"anavari",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,anavari_dimid/),anavari_varid ))
   call nccheck( "a83fillvalue",nf90_put_att(lunit,anavari_varid ,"_FillValue",cmiss) )
   call nccheck( "a83missing_value",nf90_put_att(lunit,anavari_varid ,"missing_value",cmiss) )

   call nccheck( "a84" ,nf90_def_var(lunit,"volanavari",NF90_INT,&
        (/ana_dimid,anavari_dimid,network_dimid/),volanavari_varid ))
   call nccheck( "a84fillvalue",nf90_put_att(lunit,volanavari_varid  ,"_FillValue",imiss) )
   call nccheck( "a84missing_value",nf90_put_att(lunit,volanavari_varid  ,"missing_value",imiss) )
end if
if (nanavard > 0) then
   call nccheck( "a85" ,nf90_def_var(lunit,"anavard",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,anavard_dimid/),anavard_varid ))
   call nccheck( "a85fillvalue",nf90_put_att(lunit,anavard_varid ,"_FillValue",cmiss) )
   call nccheck( "a85missing_value",nf90_put_att(lunit,anavard_varid ,"missing_value",cmiss) )

   call nccheck( "a86" ,nf90_def_var(lunit,"volanavard",NF90_DOUBLE,&
        (/ana_dimid,anavard_dimid,network_dimid/),volanavard_varid ))
   call nccheck( "a86fillvalue",nf90_put_att(lunit,volanavard_varid  ,"_FillValue",dmiss) )
   call nccheck( "a86missing_value",nf90_put_att(lunit,volanavard_varid  ,"missing_value",dmiss) )
end if
if (nanavarb > 0) then
   call nccheck( "a87" ,nf90_def_var(lunit,"anavarb",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,anavarb_dimid/),anavarb_varid ))
   call nccheck( "a87fillvalue",nf90_put_att(lunit,anavarb_varid ,"_FillValue",cmiss) )
   call nccheck( "a87missing_value",nf90_put_att(lunit,anavarb_varid ,"missing_value",cmiss) )

   call nccheck( "a88" ,nf90_def_var(lunit,"volanavarb",NF90_BYTE,&
        (/ana_dimid,anavarb_dimid,network_dimid/),volanavarb_varid ))
   call nccheck( "a88fillvalue",nf90_put_att(lunit,volanavarb_varid  ,"_FillValue",ibmiss) )
   call nccheck( "a88missing_value",nf90_put_att(lunit,volanavarb_varid  ,"missing_value",ibmiss) )
end if
if (nanavarc > 0) then 
   call nccheck( "a89" ,nf90_def_var(lunit,"anavarc",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,anavarc_dimid/),anavarc_varid ))
   call nccheck( "a89fillvalue",nf90_put_att(lunit,anavarc_varid ,"_FillValue",cmiss) )
   call nccheck( "a89missing_value",nf90_put_att(lunit,anavarc_varid ,"missing_value",cmiss) )

   call nccheck( "a90" ,nf90_def_var(lunit,"volanavarc",NF90_CHAR,&
        (/anavarc_len_dimid,ana_dimid,anavarc_dimid,network_dimid/),volanavarc_varid ))
   call nccheck( "a90fillvalue",nf90_put_att(lunit,volanavarc_varid  ,"_FillValue",cmiss) )
   call nccheck( "a90missing_value",nf90_put_att(lunit,volanavarc_varid  ,"missing_value",cmiss) )
end if



! dati

if (ndativarr > 0) then
   call nccheck( "d81" ,nf90_def_var(lunit,"dativarr",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,dativarr_dimid/),dativarr_varid ))
   call nccheck( "d8fillvalue",nf90_put_att(lunit,dativarr_varid, "_FillValue",cmiss) )
   call nccheck( "d8missing_value",nf90_put_att(lunit,dativarr_varid, "missing_value",cmiss) )

   call nccheck( "d82" ,nf90_def_var(lunit,"voldativarr",NF90_REAL,&
        (/ana_dimid,time_dimid,level_dimid,timerange_dimid,dativarr_dimid,network_dimid/),voldativarr_varid ))
   call nccheck( "d82fillvalue",nf90_put_att(lunit,voldativarr_varid , "_FillValue",rmiss) )
   call nccheck( "d82missing_value",nf90_put_att(lunit,voldativarr_varid , "missing_value",rmiss) )
end if
if (ndativari > 0) then
   call nccheck( "d83" ,nf90_def_var(lunit,"dativari",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,dativari_dimid/),dativari_varid ))
   call nccheck( "d83fillvalue",nf90_put_att(lunit,dativari_varid, "_FillValue",cmiss) )
   call nccheck( "d83missing_value",nf90_put_att(lunit,dativari_varid, "missing_value",cmiss) )

   call nccheck( "d84" ,nf90_def_var(lunit,"voldativari",NF90_INT,&
        (/ana_dimid,time_dimid,level_dimid,timerange_dimid,dativari_dimid,network_dimid/),voldativari_varid ))
   call nccheck( "d84fillvalue",nf90_put_att(lunit,voldativari_varid , "_FillValue",imiss) )
   call nccheck( "d84missing_value",nf90_put_att(lunit,voldativari_varid , "missing_value",imiss) )
end if
if (ndativard > 0) then
   call nccheck( "d85" ,nf90_def_var(lunit,"dativard",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,dativard_dimid/),dativard_varid ))
   call nccheck( "d85fillvalue",nf90_put_att(lunit,dativard_varid, "_FillValue",cmiss) )
   call nccheck( "d85missing_value",nf90_put_att(lunit,dativard_varid, "missing_value",cmiss) )

   call nccheck( "d86" ,nf90_def_var(lunit,"voldativard",NF90_DOUBLE,&
        (/ana_dimid,time_dimid,level_dimid,timerange_dimid,dativard_dimid,network_dimid/),voldativard_varid ))
   call nccheck( "d86fillvalue",nf90_put_att(lunit,voldativard_varid , "_FillValue",dmiss) )
   call nccheck( "d86missing_value",nf90_put_att(lunit,voldativard_varid , "missing_value",dmiss) )
end if
if (ndativarb > 0) then
   call nccheck( "d87" ,nf90_def_var(lunit,"dativarb",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,dativarb_dimid/),dativarb_varid ))
   call nccheck( "d87fillvalue",nf90_put_att(lunit,dativarb_varid, "_FillValue",cmiss) )
   call nccheck( "d87missing_value",nf90_put_att(lunit,dativarb_varid, "missing_value",cmiss) )

   call nccheck( "d88" ,nf90_def_var(lunit,"voldativarb",NF90_BYTE,&
        (/ana_dimid,time_dimid,level_dimid,timerange_dimid,dativarb_dimid,network_dimid/),voldativarb_varid ))
   call nccheck( "d88fillvalue",nf90_put_att(lunit,voldativarb_varid , "_FillValue",ibmiss) )
   call nccheck( "d88missing_value",nf90_put_att(lunit,voldativarb_varid , "missing_value",ibmiss) )
end if
if (ndativarc > 0) then
   call nccheck( "d89" ,nf90_def_var(lunit,"dativarc",NF90_CHAR,&
        (/var_len_dimid,var_vdim_dimid ,dativarc_dimid/),dativarc_varid ))
   call nccheck( "d89fillvalue",nf90_put_att(lunit,dativarc_varid, "_FillValue",cmiss) )
   call nccheck( "d89missing_value",nf90_put_att(lunit,dativarc_varid, "missing_value",cmiss) )

   call nccheck( "d90" ,nf90_def_var(lunit,"voldativarc",NF90_CHAR,&
        (/dativarc_len_dimid,ana_dimid,time_dimid,level_dimid,timerange_dimid,dativarc_dimid,network_dimid/),voldativarc_varid ))
   call nccheck( "d90fillvalue",nf90_put_att(lunit,voldativarc_varid , "_FillValue",cmiss) )
   call nccheck( "d90missing_value",nf90_put_att(lunit,voldativarc_varid , "missing_value",cmiss) )
end if

! end definition
call nccheck("22", nf90_enddef(lunit) )


if (associated(this%ana))       call nccheck("23", nf90_put_var(lunit, ana_lat_varid, getlat(this%ana(:)%coord)))
if (associated(this%ana))       call nccheck("24", nf90_put_var(lunit, ana_lon_varid, getlon(this%ana(:)%coord)))

if (associated(this%ana))       call nccheck("25", nf90_put_var(lunit, ana_ident_varid, this%ana(:)%ident))

if (associated(this%time))      call nccheck("26", nf90_put_var(lunit, time_iminuti_varid   , &
 int(timedelta_getamsec(this%time-timeref)/60000)))

if (associated(this%level)) then
  do i=1,nlevel

    call nccheck("27", nf90_put_var(lunit, level_vect_varid,&
     (/this%level(i)%level1,&
     this%level(i)%l1,&
     this%level(i)%level2,&
     this%level(i)%l2/),&
     start=(/1,i/),count=(/4,1/)))
  end do
end if

if (associated(this%timerange)) then
  do i=1,ntimerange
    call nccheck( "28",nf90_put_var(lunit, timerange_vect_varid,&
     (/this%timerange(i)%timerange,&
     this%timerange(i)%p1,&
     this%timerange(i)%p2/),&
     start=(/1,i/),count=(/3,1/)))
  end do
end if

if (associated(this%network)) then
    call nccheck( "29",nf90_put_var(lunit, network_name_varid,this%network(:)%name))
end if


! ana

do i=1,nanavarr
   if (associated(this%anavar%r)) then
      varchar(1)= this%anavar%r(i)%description
      varchar(2)= this%anavar%r(i)%unit
      varchar(3)= this%anavar%r(i)%btable
      call nccheck( "a291",nf90_put_var(lunit, anavarr_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,nanavari
   if (associated(this%anavar%i)) then
      varchar(1)= this%anavar%i(i)%description
      varchar(2)= this%anavar%i(i)%unit
      varchar(3)= this%anavar%i(i)%btable
      call nccheck( "a292",nf90_put_var(lunit, anavari_varid,&
      varchar&
      ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,nanavard
   if (associated(this%anavar%d)) then
      varchar(1)= this%anavar%d(i)%description
      varchar(2)= this%anavar%d(i)%unit
      varchar(3)= this%anavar%d(i)%btable
      call nccheck( "a293",nf90_put_var(lunit, anavard_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,nanavarb
   if (associated(this%anavar%b)) then
      varchar(1)= this%anavar%b(i)%description
      varchar(2)= this%anavar%b(i)%unit
      varchar(3)= this%anavar%b(i)%btable
      call nccheck( "a294",nf90_put_var(lunit, anavarb_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,nanavarc
   if (associated(this%anavar%c)) then
      varchar(1)= this%anavar%c(i)%description
      varchar(2)= this%anavar%c(i)%unit
      varchar(3)= this%anavar%c(i)%btable
      call nccheck( "a295",nf90_put_var(lunit, anavarc_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do



! dati

do i=1,ndativarr
   if (associated(this%dativar%r)) then
     varchar(1)=this%dativar%r(i)%description
     varchar(2)=this%dativar%r(i)%unit
     varchar(3)=this%dativar%r(i)%btable
      call nccheck( "a291",nf90_put_var(lunit, dativarr_varid,&
       varchar&
      ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,ndativari
   if (associated(this%dativar%i)) then
     varchar(1)=this%dativar%i(i)%description
     varchar(2)=this%dativar%i(i)%unit
     varchar(3)=this%dativar%i(i)%btable
      call nccheck( "a292",nf90_put_var(lunit, dativari_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,ndativard
   if (associated(this%dativar%d)) then
     varchar(1)=this%dativar%d(i)%description
     varchar(2)=this%dativar%d(i)%unit
     varchar(3)=this%dativar%d(i)%btable
      call nccheck( "a293",nf90_put_var(lunit, dativard_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,ndativarb
   if (associated(this%dativar%b)) then
     varchar(1)=this%dativar%b(i)%description
     varchar(2)=this%dativar%b(i)%unit
     varchar(3)=this%dativar%b(i)%btable
      call nccheck( "a294",nf90_put_var(lunit, dativarb_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do

do i=1,ndativarc
   if (associated(this%dativar%c)) then
     varchar(1)=this%dativar%c(i)%description
     varchar(2)=this%dativar%c(i)%unit
     varchar(3)=this%dativar%c(i)%btable
      call nccheck( "a295",nf90_put_var(lunit, dativarc_varid,&
       varchar&
       ,start=(/1,1,i/),count=(/65,3,1/)))
   end if
end do


!!$if (associated(this%volanar))      write(unit=lunit)this%volanar
if (associated(this%volanar))call nccheck("a231", nf90_put_var(lunit,volanavarr_varid,this%volanar))
if (associated(this%volanai))call nccheck("a232", nf90_put_var(lunit,volanavari_varid,this%volanai))
if (associated(this%volanad))call nccheck("a233", nf90_put_var(lunit,volanavard_varid,this%volanad))
if (associated(this%volanab))call nccheck("a234", nf90_put_var(lunit,volanavarb_varid,this%volanab))
if (associated(this%volanac))call nccheck("a235", nf90_put_var(lunit,volanavarc_varid,this%volanac))


if (associated(this%voldatir))call nccheck("d231", nf90_put_var(lunit,voldativarr_varid,this%voldatir))
if (associated(this%voldatii))call nccheck("d232", nf90_put_var(lunit,voldativari_varid,this%voldatii))
if (associated(this%voldatid))call nccheck("d233", nf90_put_var(lunit,voldativard_varid,this%voldatid))
if (associated(this%voldatib))call nccheck("d234", nf90_put_var(lunit,voldativarb_varid,this%voldatib))
if (associated(this%voldatic))call nccheck("d235", nf90_put_var(lunit,voldativarc_varid,this%voldatic))

if (.not. present(ncunit)) call nccheck("90", nf90_close(lunit) )

end subroutine vol7d_netcdf_export



subroutine vol7d_netcdf_export_CF (this,ncconventions,ncunit,description,filename)

TYPE(vol7d),INTENT(IN) :: this !< volume vol7d da scrivere 
integer,optional,intent(inout) :: ncunit !< unit netcdf su cui scrivere; se passata =0 ritorna il valore rielaborato (default =elaborato internamente da netcdf )
character(len=*),intent(in) :: ncconventions !< tipo di convenzione da utilizzare nella scrittura netcdf (per ora supportato "CF-1.1 vol7d")
character(len=*),intent(inout),optional :: filename !< nome del file su cui scrivere; se passato ="" ritorna il valore rielaborato
character(len=*),INTENT(IN),optional :: description !< descrizione del volume

integer :: lunit
character(len=254) :: ldescription,arg,lfilename
integer :: nana, ntime, ntimerange, nlevel, nnetwork, &
 ndativarr, ndativari, ndativarb, ndativard, ndativarc,&
 ndatiattrr, ndatiattri, ndatiattrb, ndatiattrd, ndatiattrc,&
 ndativarattrr, ndativarattri, ndativarattrb, ndativarattrd, ndativarattrc,&
 nanavarr, nanavari, nanavarb, nanavard, nanavarc,&
 nanaattrr, nanaattri, nanaattrb, nanaattrd, nanaattrc,&
 nanavarattrr, nanavarattri, nanavarattrb, nanavarattrd, nanavarattrc
!integer :: im,id,iy
integer :: tarray(8)
logical :: opened,exist

integer :: ana_ident_varid,ana_dimid,ana_lon_varid,ana_lat_varid &
 ,ident_len_dimid,var_len_dimid &
 ,level_dimid,level_vdim_dimid,level_vect_varid,network_dimid,network_id_varid,timerange_vdim_dimid &
 ,time_iminuti_varid,time_dimid,timerange_dimid,timerange_vect_varid &
 ,dativard_dimid,dativarr_dimid,dativari_dimid,dativarb_dimid,dativarc_len_dimid,dativarc_dimid &
 ,dativarr_varid ,dativari_varid ,dativard_varid ,dativarb_varid ,dativarc_varid &
 ,anavard_dimid,anavarr_dimid,anavari_dimid,anavarb_dimid,anavarc_len_dimid,anavarc_dimid &
 ,anavarr_varid ,anavari_varid ,anavard_varid ,anavarb_varid ,anavarc_varid &
 ,anavarr_btable_varid,anavari_btable_varid,anavard_btable_varid,anavarb_btable_varid,anavarc_btable_varid &
 ,dativarr_btable_varid,dativari_btable_varid,dativard_btable_varid,dativarb_btable_varid,dativarc_btable_varid


integer :: i

type(datetime) :: timeref 

integer :: category
character(len=512):: a_name


call l4f_launcher(a_name,a_name_append=subcategory)
category=l4f_category_get(a_name)


if (ncconventions /= "CF-1.1") then

  call l4f_category_log(category,L4F_INFO,"ncconventions not supported: "//TRIM(ncconventions))
  call exit(1)
end if


end subroutine vol7d_netcdf_export_CF



subroutine nccheck(stringa,status)
integer, intent ( in) :: status
character (len=*) :: stringa

if(status /= nf90_noerr) then
  print *, stringa
  print *, trim(nf90_strerror(status))
  stop "Stopped"
end if
end subroutine nccheck


end MODULE vol7d_netcdf_class
